//******************************************************************************
//
//      PWM_SW_6P_LED.C
//
//
//      CONTROLE DE POTENCIA CARGA CC (METODO  PWM) EM 6 PASSOS
//
//      COM PIC 12F675
//
//      Utiliza 1 chave dactilar para controle.
//      Frequncia PWM de aproximadamente 1khz
//
//      Autor: Cludio Lrios
//      Data: 18/11/2015
//
//
//
//
//      PINAGENS:
//      pino 1 - +5v
//      pino 2 - LEDS: 3 CAT,4 ANODO,5 CAT
//      pino 3 - disco piezoeletrico ao gnd (beep)
//      pino 4 - chave dactilar ao gnd
//      pino 5 - sada pwm a um resistor de 470 a base do transistor NPN. Emissor
//               ao gnd e coletor a carga a ser controlada para o VCC.
//      pino 6 - LEDS : 1 CAT, 2 ANODO, 3 ANODO, 4 CAT
//      pino 7 - LEDS : 1 ANODO, 2 CAT, 5 ANODO
//
//      Compilado com CCS C Compiler
//
//      Este arquivo  parte integrante do blog "LRIOS.TECNOLOGIA.WS'.
//      Obs. apenas para fins didticos.
//==============================================================================


#include <12F675.h>
#device adc=8
#use delay(clock=4000000)// OSC INTERNO DE 4MHZ
#fuses NOWDT,INTRC_IO, NOCPD, NOPROTECT, NOMCLR, PUT, BROWNOUT //CONFIG WORD

//bytes

#byte  gpio      = 0x05
#byte  trisio    = 0x85
#byte  tmr0      = 0x01
#byte  wpu       = 0x95

//bits
#bit   gwpu      = 0x81.7 //geral pullups
#bit   out       = 0x5.2 //saida pwm
#bit   sw_sel    = 0x5.3 //sw aumenta percentual ligado
#bit   beep      = 0x5.4 //saida de som para disco piezoeletrico
#bit   TRISIO0   = 0X85.0
#bit   TRISIO1   = 0X85.1
#bit   TRISIO5   = 0X85.5
#bit    GPIO0    = 0X5.0
#bit    GPIO1    = 0X5.1
#bit    GPIO5    = 0X5.5

//defines
#define leds_off  0b101011

//VARIVEIS GLOBAIS

int8 cont,pwm,vez,passo;
int1 f_tr,f_bip;

/*******************************************************************************
           INTERRUPO DO TIMER 0 (GERAO DE PWM)
*******************************************************************************/
#int_timer0
void tmr_isr(){//GERA PWM NA SAIDA
   if(!vez){
    pwm=cont;
    tmr0=~pwm;
    vez++;
    if(pwm==2){ out=0;}else {out=1;}
    }
    else
   {
    tmr0=pwm;
    vez=0;
    if(pwm==254) {out=1;} else{out=0;}
   }
}

//==============================================================================
//           CONFIGURAES
//==============================================================================

void main() {

   setup_adc_ports(NO_ANALOGS|VSS_VDD);//DESLIGA COMPARADORES
   setup_adc(ADC_OFF);
   setup_counters(RTCC_INTERNAL,RTCC_DIV_4);//TIMER 0
   setup_timer_1(T1_DISABLED); //DESABILITA TIMER 1
   setup_vref(FALSE);//DESLIGA VREF
   setup_comparator(NC_NC_NC_NC);//DESLIGA COMPARADORES
   //wpu=0b110000;//pullup
   gpio=0;//ZERA GPIO
   trisio= 0b101011; //ACERTA DIREO DOS PORTS
   vez=0;
   f_tr=0;
   cont=2;
   passo=0;
   //gwpu=0;//geral dos pullups
   enable_interrupts(GLOBAL);//liga geral das interrupes
   enable_interrupts(INT_TIMER0);//liga interrupo do tmr0
//==============================================================================
//    INICIO DO PROGRAMA
//==============================================================================

   for(;;){//LOOP INFINITO

    if((sw_sel)&&(f_tr)){ f_tr=0;}

    if((!sw_sel)&&(!f_tr)){  //testa chave de seleo
      passo++;
      TRISIO=leds_off;//desliga todos os leds
      GPIO=GPIO&0b011100;
      if(passo==6) {passo=0; cont=2; }  //0%

      if(passo==1) {cont=51;   TRISIO0=0; TRISIO1=0; GPIO0=1; GPIO1=0;  }           //20%
      if(passo==2) {cont=102;  TRISIO0=0; TRISIO1=0; GPIO0=0; GPIO1=1;  }         //40%
      if(passo==3) {cont=153;  TRISIO5=0; TRISIO1=0; GPIO5=0; GPIO1=1;  }      //60%
      if(passo==4) {cont=204;  TRISIO5=0; TRISIO1=0; GPIO5=1; GPIO1=0;  }         //80%
      if(passo==5) {cont=254;  TRISIO5=0; TRISIO0=0; GPIO0=1; GPIO5=0;  }        //100%
      f_tr=1;//trava
      f_bip=1;//bipar

    }

    if(f_bip) {
    int16 a;

      for(a=0;a<600;a++){
       beep=1;//liga sada do beep
       delay_us(96);//delay
       beep=0; //desliga sada do beep
       delay_us(96);//delay
      }
      f_bip=0;//reseta flag
    }
     delay_ms(100);//delay

   }//for(;;)
}//main
