//******************************************************************************
//
//      PWM_SW_6P_LED.C
//
//
//      CONTROLE DE POTENCIA CARGA CC (METODO  PWM) EM 6 PASSOS
//
//      COM PIC 12F675
//
//      Utiliza apenas uma chave dactilar para controle.
//      Frequncia PWM de aproximadamente 1khz.
//      Controla a sada em 6 passos: 0%,20%,40%,60%,80% e 100%
//      Acrescentado 5 leds para indicar qual velocidade foi acionada
//      Adicionado disco piezoeltrico para gerar 'bip' ao apertar interruptor
//
//      
//      Autor: Cludio Lrios
//      Data: 20/11/2015
//
//      Aplicao: Controle de velocidade em motores CC.
//
//
//      PINAGENS DO PIC 12F675:
//      pino 1 - +5v
//      pino 2 - LEDS: 3 CAT,4 ANODO,5 CAT
//      pino 3 - disco piezoeletrico ao gnd (beep)
//      pino 4 - chave dactilar ao gnd
//      pino 5 - sada pwm a um resistor de 470 a base do transistor NPN. Emissor
//               ao gnd e coletor a carga a ser controlada para o VCC.
//      pino 6 - LEDS : 1 CAT, 2 ANODO, 3 ANODO, 4 CAT
//      pino 7 - LEDS : 1 ANODO, 2 CAT, 5 ANODO
//      pino 8 - gnd

//     
//      Compilado com MPLABX (XC8 Compiler)
//
//      Este arquivo  parte integrante do blog "LRIOS.TECNOLOGIA.WS'.
//      Obs. apenas para fins didticos.
//==============================================================================

#include <stdio.h>
#include <stdlib.h>
#include <xc.h>
#define _XTAL_FREQ  4000000  //4MHZ INTERNO
// CONFIG
#pragma config FOSC = INTRCIO   // Oscillator Selection bits (INTOSC oscillator: I/O function on GP4/OSC2/CLKOUT pin, I/O function on GP5/OSC1/CLKIN)
#pragma config WDTE = OFF       // Watchdog Timer Enable bit (WDT disabled)
#pragma config PWRTE = ON       // Power-Up Timer Enable bit (PWRT enabled)
#pragma config MCLRE = OFF      // GP3/MCLR pin function select (GP3/MCLR pin function is digital I/O, MCLR internally tied to VDD)
#pragma config BOREN = ON       // Brown-out Detect Enable bit (BOD enabled)
#pragma config CP = OFF         // Code Protection bit (Program Memory code protection is disabled)
#pragma config CPD = OFF        // Data Code Protection bit (Data memory code protection is disabled)



#define out  GP2//saida pwm
#define sw_sel  GP3//sw seleo de velocidades
#define  beep   GP4// sada para piezo eletrico (beep)
#define leds_off  0b101011//desligando leds 

//VARIVEIS GLOBAIS

char cont,pwm,vez,passo;
bit f_tr, f_bip;


void interrupt tmr_isr(){//GERA PWM NA SAIDA
   T0IF=0;
   if(!vez){
    pwm=cont;
    TMR0=~pwm;
    vez++;
    if(pwm==2){ out=0;}else {out=1;}
    }
    else
   {
    TMR0=pwm;
    vez=0;
    if(pwm==254) {out=1;} else{out=0;}
   }
}

//==============================================================================
//           CONFIGURAES
//==============================================================================

int main(int argc, char** argv) {
   CMCON=7; //Desliga comparadores
   ANSEL=0; //Desliga a/d
   OPTION_REG=0X01; //TMR0 /4  
  
   WPU=0b000000;//pullup
   GPIO=0;//ZERA GPIO
   TRISIO= 0b101011; //ACERTA DIREO DOS PORTS
   vez=0;
   f_tr=0;
   f_bip=0;
   cont=2;
   passo=0;
   GIE=1;//liga geral das interrupes
   T0IE=1;;//liga interrupo do tmr0
//==============================================================================
//    INICIO DO PROGRAMA
//==============================================================================

   for(;;){//LOOP INFINITO
   
    if((sw_sel)&&(f_tr)){ f_tr=0;}

    if((!sw_sel)&&(!f_tr)){  //testa chave de seleo
      passo++;
      TRISIO=leds_off;//desliga todos os leds
      GPIO=GPIO&0b011100;
      if(passo==6) {passo=0; cont=2; }  //0%
     
      if(passo==1) {cont=51;   TRISIO0=0; TRISIO1=0; GPIO0=1; GPIO1=0;  } //20%
      if(passo==2) {cont=102;  TRISIO0=0; TRISIO1=0; GPIO0=0; GPIO1=1;  } //40%
      if(passo==3) {cont=153;  TRISIO5=0; TRISIO1=0; GPIO5=0; GPIO1=1;  } //60%
      if(passo==4) {cont=204;  TRISIO5=0; TRISIO1=0; GPIO5=1; GPIO1=0;  } //80%
      if(passo==5) {cont=254;  TRISIO5=0; TRISIO0=0; GPIO0=1; GPIO5=0;  } //100%
      f_tr=1;//trava
      f_bip=1;//bipar

    }

    if(f_bip) {
    int a;

      for(a=0;a<600;a++){
       beep=1;//liga sada do beep
       __delay_us(96);//delay
       beep=0; //desliga sada do beep
       __delay_us(96);//delay
      }
      f_bip=0;//reseta flag
    }
     __delay_ms(100);//delay
     
   }//for(;;)
   
   return (EXIT_SUCCESS);
}//main
